<?php
mangen.php - HTML & HTMLHelp Manual Generator
Version 1.0.0
Copyright 2001-2002, Steve Blinch 
http://code.blitzaffe.com


This script was created to allow one of our clients to simultaneously maintain
an online HTML version of his helpfiles on his website, and an identical
Windows helpfile (in HTMLHelp format) to distribute with his application.

This script takes a single plain textfile as input, and creates two separate
sets of output files: one is a set of HTML files which can be displayed on a
website; the other is a set of HTMLHelp source files, which can be compiled
with Microsoft's HTMLHelp compiler.

For those who don't know what HTMLHelp is, load up NotePad (or practically
any other Windows application) and press F1 - the helpfile you see was created
from a set of HTMLHelp source files, just like the ones this script produces.


Note: This script is the product of much trial and error, and really isn't the
cleanest code.  It was intended as a quick 'n dirty solution to do precisely
what we needed it to do, and nothing more.  We weren't planning on releasing it
to the public, so lots of stuff is hardcoded which probably shouldn't be (i.e.,
you'll probably want to change some of it).

Hopefully you're comfortable reading and editing PHP code; if not, this might
not be for you, and you might want to look for a different solution.


BTW, some of you might think that this would be better handled with a desktop
application, not a PHP script.  We created it as a PHP script for two reasons:

1. So that we can make modifications & upgrades as necessary, without having
to distribute updated software to our client.
2. So that the generated HTML documents immediately appear on the clients'
website, without user intervention.  Then the client just has to download the
HTMLHelp files to compile his helpfile.

Plus, I just love PHP. :)

--------------------------------------------------------------------------------

Usage:

1. Change the variables below, using appropriate paths and names.
2. Open the sample textfile "yourappmanual.txt" and put your content into it.
3. Then just run this script any time you want to update the HTML files/HTMLHelp
Source files.  Simple as that.



$document_name = "My Application's Manual";							// document name

$input_path = "/www/yourapp/manual/";										// path to input file
$input_file = "yourappmanual.txt";											// name of input (plain text) file
$input_php_header = "header.php";												// PHP header file
$input_php_footer = "footer.php";												// PHP footer file
$input_html_header = "header.html";											// HTML header file (HTMLHelp)
$input_html_footer = "footer.html";											// HTML footer file (HTMLHelp)

$output_path = "/www/yourapp/www/manual/";							// path for PHP output files
$output_htmlhelp = "help/";															// subdirectory (under PHP output path) for HTMLHelp files
$output_name = "yourapp";																// base filename for output files

$output_ext = "php";


// get php header
$tf = fopen($input_path.$input_php_header,"r");
$php_header = fread($tf,filesize ($input_path.$input_php_header));
fclose($tf);

// get php footer
$tf = fopen($input_path.$input_php_footer,"r");
$php_footer = fread($tf,filesize ($input_path.$input_php_footer));
fclose($tf);

// get html header
$tf = fopen($input_path.$input_html_header,"r");
$html_header = fread($tf,filesize ($input_path.$input_html_header));
fclose($tf);

// get html footer
$tf = fopen($input_path.$input_html_footer,"r");
$html_footer = fread($tf,filesize ($input_path.$input_html_footer));
fclose($tf);


// open source file
$srcfile = fopen($input_path.$input_file, "r");

// reset section number counters
$section_number = array(0,0,0,0,0,0);
$curent_level = 0;
$toc_array = array();
$toc_cnt = 0;
$last_br = 0;

// proceed only if the source file was opened successfully
if ($srcfile) {
    
	$endafile = 0;
	
	// parse source file - first iteration just counts lines and gets the section headers
	while (!feof($srcfile)) {
	
	  // get one line
		$buf = fgets($srcfile,4096);
		
		// determine whether or not it's a header definition
		$n = sscanf($buf,"[h%d] %s",&$php_header_level,$title);
		
		$title .= substr($buf,strlen($php_header_level)+strlen($title)+4);
		
		if (feof($srcfile)) { 
			$endafile=1;
			$n=1; 
			$php_header_level = 0;
			$title = "End of Manual";
		}

		if ($n) {
		  
		  // yup, it is - update section numbers
			$current_level = $php_header_level;
			$section_number[$current_level]++;
			for ($i=$current_level+1; $i<6; $i++) {
				$section_number[$i]=0;
			}
			
			// generate dotted version number (i.e., 1.2.4)
			$sec="";
			
			for ($i=0; $i<=$current_level; $i++) {
				$sec.=$section_number[$i].".";
			}

			$man_name = $output_path.$output_name.".".$sec."php";
			$hh_man_name = $output_path.$output_htmlhelp.$output_name.".".$sec."html";

			if (!$endafile) {
				$toc_cnt++;
				$toc_array[$toc_cnt-1]["section"]=$sec;
				$toc_array[$toc_cnt-1]["level"]=$current_level;
				$toc_array[$toc_cnt-1]["title"]=$title;
				$toc_array[$toc_cnt-1]["file"]=$man_name;
				$toc_array[$toc_cnt-1]["html"]=$hh_man_name;
				$toc_array[$toc_cnt-1]["lines"]=0;
			}
			
		} else {
			$toc_array[$toc_cnt-1]["lines"]++;
		}
	}

	// reset source file & section values
	fseek($srcfile,0);
	
	$section_number = array(0,0,0,0,0,0);
	$curent_level = 0;
	$toc_cur = 0;
	$attr_bold = 0;
	$attr_jump = 0;
	$attr_link = 0;

	// parse source file - second iteration creates manual pages
	while (!feof($srcfile)) {
	
	  // get one line
		$buf = fgets($srcfile,4096);
		
		// determine whether or not it's a header definition
		$n = sscanf($buf,"[h%d] %s",&$php_header_level,$title);
		
		$title .= substr($buf,strlen($php_header_level)+strlen($title)+4);
		
		if (feof($srcfile)) { 
			$n=1; 
			$php_header_level = 0;
			$title = "End of Manual";
		}
		
		if ($n) {
		  
		  // yup, it is - update section numbers
		  $last_level = $current_level;
			$current_level = $php_header_level;
			$section_number[$current_level]++;
			for ($i=$current_level+1; $i<6; $i++) {
				$section_number[$i]=0;
			}
			
			// generate dotted version number (i.e., 1.2.4)
			$sec="";
			
			for ($i=0; $i<=$current_level; $i++) {
				$sec.=$section_number[$i].".";
			}

			$man_name = $output_path.$output_name.".".$sec."php";
			$hh_man_name = $output_path.$output_htmlhelp.$output_name.".".$sec."html";

			$toc_cur++;
			
			if ($manpage) {
			
				$ot = "";
				
				if ($toc_cur!=$toc_cnt+1) {
					if ($subsec) {
						//$subsec = "<b><u>Subsections</u></b><br><br>$subsec\n";
						$subsec = "<hr size=1><br>$subsec\n";
						$ot.=$subsec;
					}
			
					$ot.="<p>Next: <b><a href=\"".basename($man_name)."\">$title</a></b></p>";
					
				}

				fwrite($manpage,$ot."<p><b><a href=\"".$output_name."-toc.".$output_ext."\">Contents</a></b></p>".$php_footer);
				fclose($manpage);

				if ($hhmanpage) {
					fwrite($hhmanpage,$ot.$html_footer);
					fclose($hhmanpage);
				}
			
			}

			$subsec = "";

			// if it's not the "End of Manual" dummy section
			if ($toc_cur!=$toc_cnt+1) {
			
				// create the manual page for this section
				$manpage = fopen($man_name,"w");
				$hhmanpage = fopen($hh_man_name,"w");
				
				if ($manpage) {
				  echo "Creating manual page: <b>$man_name</b><br>\n";
				} else {
					echo "Could not create manual page<br>\n";
				}

				if ($hhmanpage) {
				  echo "Creating HTMLHelp manual page: <b>$hh_man_name</b><br>\n";
				} else {
					echo "Could not create HTMLHelp manual page<br>\n";
				}
				
				$toc_walk=$toc_cur+1;
				$subsecs = 0;
				while ($toc_array[$toc_walk-1]["level"]>$current_level) {
					if ($toc_array[$toc_walk-1]["level"]==$current_level+1) {
						$subsec .= "<a href=\"".basename($toc_array[$toc_walk-1]["file"])."\"><b>".$toc_array[$toc_walk-1]["section"]."</b> ".$toc_array[$toc_walk-1]["title"]."</a><br>\n";
						$subsecs++;
					}
					$toc_walk++;
				}
	
				if ($subsecs) { $subsecs += 3; }
				
				
				$sectitle="$sec $title";
				
				if ($manpage) {
					$ot=str_replace('%lines%',$toc_array[$toc_cur-1]["lines"]-10+$subsecs,$php_header);
					$ot=str_replace('%title%',$sectitle,$ot);
					fwrite($manpage,$ot); 
				}
				
				if ($hhmanpage) { 
					$ot=str_replace('%title%',$sectitle,$html_header);
					fwrite($hhmanpage,$ot); 
				}
				

				$lines=0;
			}

		} else {
		  $buf = str_replace("\n","",$buf);
		  $buf = str_replace("\r","",$buf);
		  
		  $maxcnt = 0;
	  	$attrpos = strpos($buf,"[b]");
		  while ($attrpos!==false) {
		  	if ($attr_bold) {
		  		$attr_bold=0;
					$b = "</b>";
		  	} else {
		  		$attr_bold=1; 
		  		$b = "<b>";
		  	}
				$buf = substr_replace($buf,$b,$attrpos,3);
				$maxcnt++;
		  	$attrpos = strpos($buf,"[b]");
		  	if ($maxcnt>100) { echo "infinite loop; aborting"; break; }
		  }
		  
		  $maxcnt = 0;
	  	$linkstart = strpos($buf,"[l]");
		  while ($linkstart!==false) {
	  		$linkend = strpos($buf,"[l]",$linkstart+3);
		  	$linkurl = substr($buf,$linkstart+3,$linkend-$linkstart-3);
		  	$buf = substr_replace($buf,"<a href=\"$linkurl\">$linkurl</a>",$linkstart,$linkend-$linkstart+3);
		  	$linkstart = strpos($buf,"[l]");
		  }
		  

			if (($buf=="") & ($lines>0) & ($last_br==0)) { $buf="<br><br>"; $last_br=1; } else { $last_br=0; }
			
			$buf .= "\n";
			
			if ($manpage) { fwrite($manpage,$buf); }
			if ($hhmanpage) { fwrite($hhmanpage,$buf); } 
			
			$lines++;
		}
	}
	
	// close source file
	fclose($srcfile);
	
  // create PHP table of contents
  $toc_name = $output_path.$output_name."-toc.php";
  $toc = fopen($toc_name,"w");
  if (!$toc) {
  	echo "Could not create table of contents!<br>";
  }
  echo "Creating table of contents: <b>$toc_name</b><br>\n";


	$ot=str_replace('%lines%',$toc_cnt+3,$php_header);
	$ot=str_replace('%title%',"Table of Contents",$ot);
  fwrite($toc,$ot);

	$numdots = 0;
	
	
	
	for ($i=0; $i<$toc_cnt; $i++) {
	  $s = $toc_array[$i]["section"];
	  $oldnumdots = $numdots;
		$numdots = substr_count ($s,'.');
		$out = "$s ".$toc_array[$i]["title"];
		switch ($numdots) {
			case 1: $out = "<u><b>$out</b></u>"; break;
			case 2: $out = "<b>$out</b>"; break;
			case 3: $out = "$out"; break;
			default: break;
		}
		
		$out="<li><a href=\"".basename($toc_array[$i]["file"])."\">$out</a>";
//		if ($i==0) { $out = "<ul>\n$out"; }
		if ($oldnumdots>$numdots) { 
			for ($j=0; $j<$oldnumdots-$numdots; $j++) {
				$out = "\n</ul>\n$out"; 
			}
		 
		 	$out = "<br><br>$out";
		 
		}
		if ($oldnumdots<$numdots) { $out = "\n<ul>$out\n";	}
		
		for ($j=0; $j<$numdots; $j++) {
			$out = " $out";
		}
		$out = "$out\n";
		fwrite($toc,$out);
	}


  // finish and close table of contents
	fwrite($toc,$php_footer);
	fclose($toc);
	



  // create HTMLHelp table of contents
  $toc_name = $output_path.$output_htmlhelp.$output_name.".hhc";
  $toc = fopen($toc_name,"w");
  if (!$toc) {
  	echo "Could not create HTMLHelp table of contents!<br>";
  }
  echo "Creating HTMLHelp table of contents<br>\n";

	$numdots = 0;
	
	$ot=
		"<!DOCTYPE HTML PUBLIC \"-//IETF//DTD HTML//EN\">\n".
		"<HTML>\n".
		"<HEAD>\n".
		"<meta name=\"GENERATOR\" content=\"Microsoft&reg; HTML Help Workshop 4.1\">\n".
		"<!-- Sitemap 1.0 -->\n".
		"</HEAD><BODY>\n".
		"<OBJECT type=\"text/site properties\">\n".
		"\t<param name=\"ImageType\" value=\"Folder\">\n".
		"</OBJECT>\n";
	fwrite($toc,$ot);
	
	
	for ($i=0; $i<$toc_cnt; $i++) {
	  $s = $toc_array[$i]["section"];
	  $oldnumdots = $numdots;
		$numdots = substr_count ($s,'.');
		$title = "$s ".$toc_array[$i]["title"];
		$title=str_replace("\n","",$title);
		$title=str_replace("\r","",$title);
		$fn = basename($toc_array[$i]["html"]);
		
		$out=str_repeat("\t",$numdots)."<LI> <OBJECT type=\"text/sitemap\">\n";
		$out.=str_repeat("\t",$numdots+1)."<param name=\"Name\" value=\"$title\">\n";
		$out.=str_repeat("\t",$numdots+1)."<param name=\"Local\" value=\"$fn\">\n";
		$out.=str_repeat("\t",$numdots+1)."</OBJECT>\n";
		
		if ($numdots<$oldnumdots) {
			$n=$oldnumdots;
			
			while ($numdots<$n) {
				$n--;
				$out=str_repeat("\t",($oldnumdots-$n))."</UL>\n$out";
			}
		}
		
		
		if ($numdots>$oldnumdots) {
			$out=str_repeat("\t",$numdots-1)."<UL>\n$out";
			
		}
		fwrite($toc,$out);
	}

	$ot="";
	for ($k=$numdots; $k>0; $k--) {
		$ot .= str_repeat("\t",$k-1)."</UL>\n";
	}
	
	$ot .= "</BODY></HTML>";

  // finish and close table of contents
	fwrite($toc,$ot);
	fclose($toc);

// -----------------------------------



	$fhhp = fopen($output_path.$output_htmlhelp.$output_name.".hhp","w");
	if ($fhhp) {
		$hhp = 
		"[OPTIONS]\n".
		"Auto Index=Yes\n".
		"Compatibility=1.1 or later\n".
		"Compiled file=$output_name.chm\n".
		"Contents file=$output_name.hhc\n".
		"Default topic=".basename($toc_array[0]["html"])."\n".
		"Display compile progress=No\n".
		"Index file=$output_name.hhk\n".
		"Language=0x409 English (United States)\n".
		"Title=$document_name\n\n\n".
		"[FILES]\n";
		
		for ($hi=0; $hi<$toc_cnt; $hi++) {
			$hhp .= basename($toc_array[$hi]["html"])."\n";
		}

		$hhp .= "\n[INFOTYPES]\n\n\n";
		
		fwrite($fhhp,$hhp);
		fclose($fhhp);
	}
	
	

}

echo "Complete";
?>
